# -*- coding: utf-8 mode: sh -*- vim:sw=4:sts=4:et:ai:si:sta:fenc=utf-8

# configuration par défaut
# doit être identique au contenu de pman.conf
# les branches sont mergées dans cet ordre:
# upstream --> develop --> [release -->] main --> dist
#      feature _/                 hotfix _/
UPSTREAM=
DEVELOP=develop
FEATURE=wip/
RELEASE=release-
MAIN=master
TAG_PREFIX=
TAG_SUFFIX=
HOTFIX=hotfix-
DIST=
NOAUTO=

CONFIG_VARS=(
    UPSTREAM DEVELOP FEATURE RELEASE MAIN TAG_PREFIX TAG_SUFFIX HOTFIX DIST NOAUTO
)

function _init_changelog() {
    setx date=date +%d/%m/%Y-%H:%M
    ac_set_tmpfile changelog
    echo >"$changelog" "\
Vérifiez et complétez la liste des changements le cas échéant.
Un fichier vide annule l'opération
Ces lignes ne seront pas incluses dans le fichier destination
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
"
}

function _filter_rel() {
    # enlever les commits "techniques" générés par ce script
    grep -v "Intégration de la branche $RELEASE" |
    grep -v "Branche $DEVELOP en version .*-SNAPSHOT"
}

function _filter_changes() {
    # enlever les commits "inutiles" pour générer le fichier CHANGES.md
    grep -vE '^([+|] )?[0-9a-f]+ modifs\.mineures sans commentaires$' |
    grep -vE '^([+|] )?[0-9a-f]+ (cosmetic|typo|bug|fix|maj projet|maj deps)\$'
}

function _format_md() {
    awk '
$0 == "" || $0 ~ /^#/ { print; next }
$1 == "+" {
  $1 = "*"
  $2 = "`" $2 "`"
  print; next
}
$1 == "|" {
  $1 = "  *"
  $2 = "`" $2 "`"
  print; next
}
{
  $1 = "* `" $1 "`"
  print; next
}
'
}

function _list_commits() {
    local source="${1:-$SrcBranch}" dest="${2:-$DestBranch}" mergebase
    setx mergebase=git merge-base "$dest" "$source"
    git log --oneline --graph "$mergebase..$source" |
        grep -vF '|\' | grep -vF '|/' | sed 's/\* //; s/^  /+ /' |
        _filter_rel
}

function _script_echo() {
    echo >>"$script"
    echo "$comment$(qvals echo "$@")" >>"$script"
}

function _script_add() {
    [ $# -gt 0 ] && _script_echo "$*"
    cat >>"$script"
}

function _push_branches() {
    local origin branch
    _rscript_echo "* push branches"
    for branch in "${push_branches[@]}"; do
        origin="$Origin"
        [ -n "$origin" ] || setx origin=git_get_branch_remote "$branch"
        [ -n "$origin" ] || origin=origin
        setx rbranch=git_get_branch_rbranch "$branch" "$origin"
        if [ -n "$rbranch" ]; then
            rbranch="${rbranch#refs/remotes/$origin/}"
            _rscript_add <<EOF
$comment$(qvals git push "$origin" "$branch:$rbranch")$or_die
EOF
        else
            _rscript_add <<EOF
$comment$(qvals git push --set-upstream "$origin" "$branch")$or_die
EOF
        fi
    done
}

function _push_tags() {
    local origin tag
    _rscript_echo "* push tags"
    for tag in "${push_tags[@]}"; do
        origin="$Origin"
        [ -n "$origin" ] || origin=origin
        _rscript_add <<EOF
$comment$(qvals git push --force "$origin" tag "$tag")$or_die
EOF
    done
}

################################################################################
# Merge

function _mscript_start() {
    >"$script"
    _script_add <<EOF
#!/bin/bash

merge=
push=
for arg in "\$@"; do
  case "\$arg" in
  merge) merge=1;;
  push) push=1;;
  esac
done
EOF
    chmod +x "$script"
}

################################################################################
# Release

function _rscript_start() {
    >"$script"
    _script_add <<EOF
#!/bin/bash

create=
merge=
push=
for arg in "\$@"; do
  case "\$arg" in
  create) create=1;;
  merge) merge=1;;
  push) push=1;;
  esac
done
EOF
    chmod +x "$script"
}

function _create_release_branch() {
    local date changelog

    _init_changelog
    echo >>"$changelog" "\
## Release $Version du $date
"
    _list_commits | _filter_changes | _format_md >>"$changelog"
    "${EDITOR:-nano}" +7 "$changelog"
    [ -s "$changelog" ] || exit_with ewarn "Création de la release annulée"

    # créer la branche de release et basculer dessus
    _script_add "* create branch $ReleaseBranch" <<EOF
$(qvals git checkout -b "$ReleaseBranch" "$SrcBranch")$or_die
EOF

    # créer le changelog
    _script_add "* update CHANGES.md" <<EOF
[ -s CHANGES.md ] && echo >>CHANGES.md
$(qvals echo "$(awk <"$changelog" '
BEGIN { p = 0 }
p == 0 && $0 == "" { p = 1; next }
p == 1 { gsub(/\$/, "\\$", $0); print }
')") >>CHANGES.md
git add CHANGES.md
EOF

    # mettre à jour la version
    _script_add "* update VERSION.txt" <<EOF
$(qvals echo "$Version") >VERSION.txt
git add VERSION.txt
EOF

    # Enregistrer les changements
    _script_add "* commit" <<EOF
$(qvals git commit -m "Init changelog & version $Version")
EOF
}

function _merge_release_branch() {
    local dest="$1" tag="$2"

    # basculer sur la branche
    _script_add "* switch to branch $dest" <<EOF
$comment$(qvals git checkout "$dest")$or_die
EOF

    # fusionner la branche de release
    _script_add "* merge branch $ReleaseBranch" <<EOF
$comment$(qvals git merge "$ReleaseBranch" -m "Intégration de la branche $ReleaseBranch" --no-ff)$or_die
EOF
    array_addu push_branches "$dest"

    # tagger la release
    if [ -n "$tag" ]; then
        _script_add "* create tag $tag" <<EOF
$comment$(qvals git tag --force "$tag")$or_die
EOF
        array_addu push_tags "$tag"
    fi
}

function _delete_release_branch() {
    _script_add "* delete branch $ReleaseBranch" <<EOF
$comment$(qvals git branch -D "$ReleaseBranch")$or_die
EOF
}
